@@ -39,6 +39,12 @@ sub getCSName {
   my ($self) = @_;
   return (defined $$self{alias} ? $$self{alias} : $$self{cs}->getCSName); }
 
+# NOTE: Need to clean up alias; really should already be Token (or Tokens?)
+# and is not always a CS!
+sub getCSorAlias {
+  my ($self) = @_;
+  return (defined $$self{alias} ? T_CS($$self{alias}) : $$self{cs}); }
+
 sub isExpandable {
   return 0; }
 
@@ -48,21 +54,45 @@ sub isRegister {
 sub isPrefix {
   return 0; }
 
+# The following come from flags, but probably(?) only make sense for Expandable (macros)
+# and furthermore, only isProtected is used for anything significant.
+sub isProtected {
+  my ($self) = @_;
+  return $$self{isProtected}; }
+
+sub setIsProtected {
+  my ($self) = @_;
+  $$self{isProtected} = 1;
+  return; }
+
+sub isOuter {
+  my ($self) = @_;
+  return $$self{isOuter}; }
+
+sub setIsOuter {
+  my ($self) = @_;
+  $$self{isOuter} = 1;
+  return; }
+
+sub isLong {
+  my ($self) = @_;
+  return $$self{isLong}; }
+
+sub setIsLong {
+  my ($self) = @_;
+  $$self{isLong} = 1;
+  return; }
+
 sub getLocator {
   my ($self) = @_;
   return $$self{locator}; }
 
 sub readArguments {
   my ($self, $gullet) = @_;
-  my $params = $self->getParameters;
-  return ($params ? $params->readArguments($gullet, $self) : ()); }
+  return ($$self{parameters} ? $$self{parameters}->readArguments($gullet, $self) : ()); }
 
 sub getParameters {
   my ($self) = @_;
-  # Allow defering these until the Definition is actually used.
-  if ((defined $$self{parameters}) && !ref $$self{parameters}) {
-    require LaTeXML::Package;
-    $$self{parameters} = LaTeXML::Package::parseParameters($$self{parameters}, $$self{cs}); }
   return $$self{parameters}; }
 
 #======================================================================
@@ -80,12 +110,6 @@ sub toString {
   return ($$self{parameters}
     ? ToString($$self{cs}) . ' ' . ToString($$self{parameters}) : ToString($$self{cs})); }
 
-# Return the Tokens that would invoke the given definition with arguments.
-sub invocation {
-  my ($self, @args) = @_;
-  my $params = $self->getParameters;
-  return ($$self{cs}, ($params ? $params->revertArguments(@args) : ())); }
-
 #======================================================================
 # Tracing support
 sub tracingCSName {
@@ -101,7 +125,12 @@ sub tracingCSName {
 sub tracingArgs {
   my ($self, @args) = @_;
   my $i = 1;
-  return join("\n", map { '#' . $i++ . '<-' . ToString($_) } @args); }
+  return join("\n", map { '#' . $i++ . '<-' . tracingArgToString($_) } @args); }
+
+# Annoying special handing of registers
+sub tracingArgToString {
+  my ($arg) = @_;
+  return (ref $arg eq 'ARRAY' ? '[' . join(',', map { ToString($_) } @$arg) . ']' : ToString($arg)); }
 
 #======================================================================
 # Profiling support
@@ -113,15 +142,15 @@ sub tracingArgs {
 # $mode = expand | digest | absorb
 sub startProfiling {
   my ($cs, $mode) = @_;
-  my $name = $cs->getCSName;
+  my $name  = $cs->getCSName;
   my $entry = $STATE->lookupMapping('runtime_profile', $name);
   # [#calls, max_depth, inclusive_time, exclusive_time, #pending
   #   starts of pending calls...]
   if (!defined $entry) {
     $entry = [0, 0, 0, 0, 0]; $STATE->assignMapping('runtime_profile', $name, $entry); }
   $$entry[0]++ unless $mode eq 'absorb';    # One more call.
-  $$entry[4]++;    # One more pending...
-                   #print STDERR "START PROFILE $mode of ".ToString($cs)."\n";
+  $$entry[4]++;                             # One more pending...
+                                            #Debug("START PROFILE $mode of ".ToString($cs));
   $STATE->pushValue('runtime_stack', [$name, $mode, [Time::HiRes::gettimeofday], $entry]);
   return; }
 
@@ -133,21 +162,23 @@ sub startProfiling {
 sub stopProfiling {
   my ($cs, $mode) = @_;
   $cs = $cs->getString if $cs->getCatcode == CC_MARKER;    # Special case for macros!!
+  return unless ref $cs;
   my $name      = $cs->getCSName;
   my $stack     = $STATE->lookupValue('runtime_stack');
   my $currdepth = scalar(@$stack);
   my $prevdepth = $STATE->lookupValue('runtime_maxdepth') || '0';
   $STATE->assignValue('runtime_maxdepth' => $currdepth, 'global') if $currdepth > $prevdepth;
+
   while (@{$stack}) {
     my ($top, $topmode, $t0, $entry) = @{ $$stack[-1] };
     if ((($top ne $name) || ($topmode ne $mode)) && ($topmode ne 'expand')) {
       return if $mode eq 'expand';                         # No error (yet) if this is a macro end marker.
-      print STDERR "\nPROFILE Error: ending $mode of $name but stack holds "
-        . join(',', map { $$_[0] . '(' . $$_[1] . ')' } @$stack) . ", $top ($topmode)\n";
+      Debug("PROFILE Error: ending $mode of $name but stack holds "
+          . join(',', map { $$_[0] . '(' . $$_[1] . ')' } @$stack) . ", $top ($topmode)");
       return; }
     pop(@$stack);
     my $duration = Time::HiRes::tv_interval($t0, [Time::HiRes::gettimeofday]);
-    my $depth = $$entry[4];
+    my $depth    = $$entry[4];
     if ($depth > $$entry[1]) {
       $$entry[1] = $depth; }
     $$entry[2] += $duration if $depth == 1;    # add to inclusive time only in uppermost call
@@ -157,7 +188,7 @@ sub stopProfiling {
       my ($callername, $callermode, $callerstart, $callerentry) = @$caller;
       $$callerentry[3] -= $duration; }         # Remove our cost from caller's exclusive time.
     return if $top eq $name; }
-  print STDERR "\nPROFILE Error: ending $mode of $name but stack is empty\n"
+  Debug("PROFILE Error: ending $mode of $name but stack is empty")
     unless $mode eq 'expand';
   return; }
 
@@ -169,7 +200,7 @@ sub showProfile {
     my @cs    = keys %$profile;
     my $calls = 0;
     map { $calls += $$profile{$_}[0] } @cs;
-    my $depth = $STATE->lookupValue('runtime_maxdepth');
+    my $depth    = $STATE->lookupValue('runtime_maxdepth');
     my @frequent = sort { $$profile{$b}[0] <=> $$profile{$a}[0] } @cs;
     @frequent = @frequent[0 .. $MAX_PROFILE_ENTRIES];
     my @deepest = sort { $$profile{$b}[1] <=> $$profile{$a}[1] } @cs;
@@ -178,21 +209,21 @@ sub showProfile {
     @inclusive = @inclusive[0 .. $MAX_PROFILE_ENTRIES];
     my @exclusive = sort { $$profile{$b}[3] <=> $$profile{$a}[3] } @cs;
     @exclusive = @exclusive[0 .. $MAX_PROFILE_ENTRIES];
-    print STDERR "\nProfiling results:\n";
-    print STDERR "Total calls: $calls; Maximum depth: $depth\n";
-    print STDERR "Most frequent:\n   "
-      . join(', ', map { $_ . ':' . $$profile{$_}[0] } @frequent) . "\n";
-    print STDERR "Deepest :\n   "
-      . join(', ', map { $_ . ':' . $$profile{$_}[1] } @deepest) . "\n";
-    print STDERR "Most expensive inclusive:\n   "
-      . join(', ', map { $_ . ':' . sprintf("%.2fs", $$profile{$_}[2]) } @inclusive) . "\n";
-    print STDERR "Most expensive exclusive:\n   "
-      . join(', ', map { $_ . ':' . sprintf("%.2fs", $$profile{$_}[3]) } @exclusive) . "\n";
+    Debug("Profiling results:");
+    Debug("Total calls: $calls; Maximum depth: $depth");
+    Debug("Most frequent:\n   "
+        . join(', ', map { $_ . ':' . $$profile{$_}[0] } @frequent));
+    Debug("Deepest :\n   "
+        . join(', ', map { $_ . ':' . $$profile{$_}[1] } @deepest));
+    Debug("Most expensive inclusive:\n   "
+        . join(', ', map { $_ . ':' . sprintf("%.2fs", $$profile{$_}[2]) } @inclusive));
+    Debug("Most expensive exclusive:\n   "
+        . join(', ', map { $_ . ':' . sprintf("%.2fs", $$profile{$_}[3]) } @exclusive));
 
     my $stack = $STATE->lookupValue('runtime_stack');
     if (@$stack) {
-      print STDERR "The following were never marked as done:\n  "
-        . join(', ', map { $$_[0] . '(' . $$_[1] . ')' } @$stack) . "\n"; } }
+      Debug("The following were never marked as done:\n  "
+          . join(', ', map { $$_[0] . '(' . $$_[1] . ')' } @$stack)); } }
   return; }
 
 #===============================================================================
