@@ -16,6 +16,7 @@ use LaTeXML::Global;
 use LaTeXML::Common::Object;
 use LaTeXML::Common::Error;
 use LaTeXML::Core::Token;
+use LaTeXML::Core::Tokens;
 use base qw(LaTeXML::Core::Definition::Expandable);
 
 # Conditional control sequences; Expandable
@@ -35,13 +36,11 @@ sub getTest {
   my ($self) = @_;
   return $$self{test}; }
 
-# Note that although conditionals are Expandable,
-# they do NOT defined as macros, so they don't need to handle doInvocation,
 sub invoke {
   my ($self, $gullet) = @_;
   # A real conditional must have condition_type set
   if (my $cond_type = $$self{conditional_type}) {
-    if ($cond_type eq 'if') {
+    if (($cond_type eq 'if') || ($cond_type eq 'unless')) {
       return $self->invoke_conditional($gullet); }
     elsif ($cond_type eq 'else') {
       return $self->invoke_else($gullet); }
@@ -61,25 +60,26 @@ sub invoke_conditional {
   local $LaTeXML::IFFRAME = { token => $LaTeXML::CURRENT_TOKEN, start => $gullet->getLocator,
     parsing => 1, elses => 0, ifid => $ifid };
   $STATE->unshiftValue(if_stack => $LaTeXML::IFFRAME);
-
-  my @args = $self->readArguments($gullet);
+  my $parms = $$self{parameters};
+  my @args  = ($parms ? $parms->readArguments($gullet) : ());
   $$LaTeXML::IFFRAME{parsing} = 0;    # Now, we're done parsing the Test clause.
-  my $tracing = $STATE->lookupValue('TRACINGCOMMANDS');
-  print STDERR '{' . $self->tracingCSName . "} [#$ifid]\n" if $tracing;
-  print STDERR $self->tracingArgs(@args) . "\n" if $tracing && @args;
+  my $tracing = $STATE->lookupValue('TRACINGCOMMANDS') || $LaTeXML::DEBUG{tracing};
+  if ($tracing) {
+    Debug('{' . $self->tracingCSName . "} [#$ifid]");
+    Debug($self->tracingArgs(@args)) if @args; }
   if (my $test = $self->getTest) {
     my $result = &$test($gullet, @args);
     if ($result) {
-      print STDERR "{true}\n" if $tracing; }
+      Debug("{true}") if $tracing; }
     else {
       my $to = skipConditionalBody($gullet, -1);
-      print STDERR "{false} [skipped to " . ToString($to) . "]\n" if $tracing; } }
+      Debug("{false} [skipped to " . ToString($to) . "]") if $tracing; } }
   # If there's no test, it must be the Special Case, \ifcase
   else {
     my $num = $args[0]->valueOf;
     if ($num > 0) {
       my $to = skipConditionalBody($gullet, $num);
-      print STDERR "{$num} [skipped to " . ToString($to) . "]\n" if $tracing; } }
+      Debug("{$num} [skipped to " . ToString($to) . "]") if $tracing; } }
   return; }
 
 #======================================================================
@@ -111,11 +111,16 @@ sub skipConditionalBody {
   my $level = 1;
   my $n_ors = 0;
   my $start = $gullet->getLocator;
-  # NOTE: Open-coded manipulation of if_stack!
+  # NOTE: Open-coded manipulation of if_stack!, Gullet and Token's
   # [we're only reading tokens & looking up, so State shouldn't change behind our backs]
   my $stack = $STATE->lookupValue('if_stack');
   while (1) {
-    my ($t, $cond_type) = $gullet->readNextConditional;
+    my ($t, $cond_type);
+    while ($t = shift(@{ $$gullet{pushback} }) || $$gullet{mouth}->readToken()) {
+      $t = $$t[2] if $$t[1] == CC_SMUGGLE_THE;
+      if ($LaTeXML::Core::State::CATCODE_ACTIVE_OR_CS[$$t[1]]
+        && ($cond_type = $STATE->lookupConditional($t))) {
+        last; } }
     last unless $cond_type;
     if ($cond_type eq 'if') {    #  Found a \ifxx of some sort
       $level++; }
@@ -126,8 +131,7 @@ sub skipConditionalBody {
       elsif (!--$level) {           # If no more nesting, we're done.
         shift(@$stack);             # Done with this frame
         return $t; } }              # AND Return the finishing token.
-    elsif ($level > 1) {            # Ignore \else,\or nested in the body.
-    }
+    elsif ($level > 1) { }                                    # Ignore \else,\or nested in the body.
     elsif (($cond_type eq 'or') && (++$n_ors == $nskips)) {
       return $t; }
     elsif (($cond_type eq 'else') && $nskips
@@ -150,7 +154,7 @@ sub invoke_else {
         . " since we seem not to be in a conditional");
     return; }
   elsif ($$stack[0]{parsing}) {     # Defer expanding the \else if we're still parsing the test
-    return [T_CS('\relax'), $LaTeXML::CURRENT_TOKEN]; }
+    return Tokens(T_CS('\relax'), $LaTeXML::CURRENT_TOKEN); }
   elsif ($$stack[0]{elses}) {       # Already seen an \else's at this level?
     Error('unexpected', $LaTeXML::CURRENT_TOKEN, $gullet,
       "Extra " . Stringify($LaTeXML::CURRENT_TOKEN),
@@ -159,10 +163,10 @@ sub invoke_else {
   else {
     local $LaTeXML::IFFRAME = $$stack[0];
     my $t = skipConditionalBody($gullet, 0);
-    print STDERR '{' . ToString($LaTeXML::CURRENT_TOKEN) . '}'
-      . " [for " . ToString($$LaTeXML::IFFRAME{token}) . " #" . $$LaTeXML::IFFRAME{ifid}
-      . " skipping to " . ToString($t) . "]\n"
-      if $STATE->lookupValue('TRACINGCOMMANDS');
+    Debug('{' . ToString($LaTeXML::CURRENT_TOKEN) . '}'
+        . " [for " . ToString($$LaTeXML::IFFRAME{token}) . " #" . $$LaTeXML::IFFRAME{ifid}
+        . " skipping to " . ToString($t) . "]")
+      if $STATE->lookupValue('TRACINGCOMMANDS') || $LaTeXML::DEBUG{tracing};
     return; } }
 
 sub invoke_fi {
@@ -174,13 +178,13 @@ sub invoke_fi {
         . " since we seem not to be in a conditional");
     return; }
   elsif ($$stack[0]{parsing}) {     # Defer expanding the \else if we're still parsing the test
-    return [T_CS('\relax'), $LaTeXML::CURRENT_TOKEN]; }
+    return Tokens(T_CS('\relax'), $LaTeXML::CURRENT_TOKEN); }
   else {                            # "expand" by removing the stack entry for this level
     local $LaTeXML::IFFRAME = $$stack[0];
     $STATE->shiftValue('if_stack');    # Done with this frame
-    print STDERR '{' . ToString($LaTeXML::CURRENT_TOKEN) . '}'
-      . " [for " . Stringify($$LaTeXML::IFFRAME{token}) . " #" . $$LaTeXML::IFFRAME{ifid} . "]\n"
-      if $STATE->lookupValue('TRACINGCOMMANDS');
+    Debug('{' . ToString($LaTeXML::CURRENT_TOKEN) . '}'
+        . " [for " . Stringify($$LaTeXML::IFFRAME{token}) . " #" . $$LaTeXML::IFFRAME{ifid} . "]")
+      if $STATE->lookupValue('TRACINGCOMMANDS') || $LaTeXML::DEBUG{tracing};
     return; } }
 
 #===============================================================================
@@ -188,7 +192,7 @@ sub invoke_fi {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
