@@ -88,26 +88,33 @@ use LaTeXML::Core::Token;    # To get CatCodes
 sub new {
   my ($class, %options) = @_;
   my $self = bless {    # table => {},
-    value => {}, meaning => {}, stash => {}, stash_active => {},
+    value   => {}, meaning  => {}, stash  => {}, stash_active => {},
     catcode => {}, mathcode => {}, sfcode => {}, lccode => {}, uccode => {}, delcode => {},
-    undo => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
+    undo    => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
     stomach => $options{stomach}, model => $options{model} }, $class;
   # Note that "100" is hardwired into TeX, The Program!!!
   $$self{value}{MAX_ERRORS} = [100];
-  $$self{value}{VERBOSITY}  = [0];
+  # Standard TeX units, in scaled points
+  $$self{value}{UNITS} = [{
+      pt => 65536, pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
+      cm => 72.27 * 65536 / 2.54, mm => 72.27 * 65536 / 2.54 / 10, dd => 1238 * 65536 / 1157,
+      cc => 12 * 1238 * 65536 / 1157, sp => 1,
+      px => 72.27 * 65536 / 72,    # Assume px=bp ?
+  }];
+
   $options{catcodes} = 'standard' unless defined $options{catcodes};
   if ($options{catcodes} =~ /^(standard|style)/) {
     # Setup default catcodes.
     my %std = ("\\" => CC_ESCAPE, "{" => CC_BEGIN, "}" => CC_END, "\$" => CC_MATH,
-      "\&" => CC_ALIGN, "\r" => CC_EOL,   "#"  => CC_PARAM, "^" => CC_SUPER,
-      "_"  => CC_SUB,   " "  => CC_SPACE, "\t" => CC_SPACE, "%" => CC_COMMENT,
-      "~" => CC_ACTIVE, chr(0) => CC_IGNORE);
+      "&" => CC_ALIGN,  "\r"   => CC_EOL,    "#"  => CC_PARAM, "^" => CC_SUPER,
+      "_" => CC_SUB,    " "    => CC_SPACE,  "\t" => CC_SPACE, "%" => CC_COMMENT,
+      "~" => CC_ACTIVE, chr(0) => CC_ESCAPE, "\f" => CC_ACTIVE);
     map { $$self{catcode}{$_} = [$std{$_}] } keys %std;
     for (my $c = ord('A') ; $c <= ord('Z') ; $c++) {
       $$self{catcode}{ chr($c) } = [CC_LETTER];
       $$self{catcode}{ chr($c + ord('a') - ord('A')) } = [CC_LETTER]; }
   }
-  $$self{value}{SPECIALS} = [['^', '_', '@', '~', '&', '$', '#', "'"]];
+  $$self{value}{SPECIALS} = [['^', '_', '~', '&', '$', '#', "'"]];
   if ($options{catcodes} eq 'style') {
     $$self{catcode}{'@'} = [CC_LETTER]; }
   $$self{mathcode}            = {};
@@ -120,10 +127,19 @@ sub new {
 
 sub assign_internal {
   my ($self, $table, $key, $value, $scope) = @_;
+  # hotcode lookupDefinition for \globaldefs,
+  # since this is called extremely often and should be highly standardized
+  if (my $globaldefs = $$self{value}{'\globaldefs'}) {
+    if (my $global_value = $$globaldefs[0][0]) {
+      # magic TeX register override: \globaldefs
+      if ($global_value == 1) {
+        $scope = 'global'; }
+      elsif ($global_value == -1) {
+        $scope = 'local'; } } }
   $scope = ($$self{prefixes}{global} ? 'global' : 'local') unless defined $scope;
   if (exists $$self{tracing_definitions}{$key}) {
-    print STDERR "ASSIGN $key in $table " . ($scope ? "($scope)" : '') . " => " .
-      (ref $value ? $value->stringify : $value) . "\n"; }
+    Debug("ASSIGN $key in $table " . ($scope ? "($scope)" : '') . " => " .
+        (ref $value ? $value->stringify : $value)); }
   if ($scope eq 'global') {
     # Remove bindings made in all frames down-to & including the next lower locked frame
     my $frame;
@@ -144,7 +160,6 @@ sub assign_internal {
       $$self{undo}[0]{$table}{$key} = 1;
       unshift(@{ $$self{$table}{$key} }, $value); } }    # And push new binding.
   else {
-    # print STDERR "Assigning $key in stash $stash\n";
     assign_internal($self, 'stash', $scope, [], 'global') unless $$self{stash}{$scope}[0];
     push(@{ $$self{stash}{$scope}[0] }, [$table, $key, $value]);
     assign_internal($self, $table, $key, $value, 'local')
@@ -312,27 +327,27 @@ sub assignDelcode {
 #======================================================================
 # Specialized versions of lookup & assign for dealing with definitions
 
-our @active_or_cs = (
+our @CATCODE_ACTIVE_OR_CS = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 1, 0, 0,
-  1, 0);
-our @letter_or_other = (
+  1, 0, 0, 0);
+our @CATCODE_LETTER_OR_OTHER = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 1,
   1, 0, 0, 0,
-  0, 0);
+  0, 0, 0, 0);
 
 # Get the `Meaning' of a token.  For active control sequence's
 # this may give the definition object (if defined) or another token (if \let) or undef
 # Any other token is returned as is.
 sub lookupMeaning {
   my ($self, $token) = @_;
-  my $e;
   if (my $cs = $token
-    && $active_or_cs[$$token[1]]
+    && $CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && !$$token[2]    # return token itself, if \noexpand
     && $$token[0]) {
     my $e = $$self{meaning}{$cs}; return $e && $$e[0]; }
   else { return $token; } }
@@ -350,13 +365,13 @@ sub assignMeaning {
 # nor cs let to executable tokens
 # This returns a definition object, or undef
 
-# merge of @executable_catcode & @PRIMITIVE_NAME
-our @executable_primitive_name = (    # [CONSTANT]
+# merge of @CATCODE_EXECUTABLE & @CATCODE_PRIMITIVE_NAME
+our @CATCODE_EXECUTABLE_PRIMITIVE_NAME = (    # [CONSTANT]
   undef,       'Begin', 'End', 'Math',
   'Align',     undef,   undef, 'Superscript',
   'Subscript', undef,   undef, undef,
   undef,       undef,   undef, undef,
-  undef,       undef);
+  undef,       undef,   undef, undef);
 
 sub lookupDefinition {
   my ($self, $token) = @_;
@@ -366,9 +381,9 @@ sub lookupDefinition {
   #  my $inmath = $self->lookupValue('IN_MATH');
   my $cc = $$token[1];
   my $lookupname =
-    ($active_or_cs[$cc]
+    ($CATCODE_ACTIVE_OR_CS[$cc]
     ? $$token[0]
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname
     && ($entry = $$self{meaning}{$lookupname})
     && ($defn  = $$entry[0])
@@ -383,14 +398,8 @@ sub lookupConditional {
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
-  my $cc = $$token[1];
-  my $lookupname =
-    ($active_or_cs[$cc]
-    ? $$token[0]
-    : $executable_primitive_name[$cc]);
-  if ($lookupname
-    && ($entry = $$self{meaning}{$lookupname})
+  if ($CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && ($entry = $$self{meaning}{ $$token[0] })
     && ($defn  = $$entry[0])
     # Can only be a token or definition; we only want defns that have conditional_type
     && ((ref $defn) ne 'LaTeXML::Core::Token')) {
@@ -403,14 +412,8 @@ sub lookupExpandable {
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
-  my $cc = $$token[1];
-  my $lookupname =
-    ($active_or_cs[$cc]
-    ? $$token[0]
-    : $executable_primitive_name[$cc]);
-  if ($lookupname
-    && ($entry = $$self{meaning}{$lookupname})
+  if ($CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && ($entry = $$self{meaning}{ $$token[0] })
     && ($defn  = $$entry[0])
     # Can only be a token or definition; we want defns!
     && ((ref $defn) ne 'LaTeXML::Core::Token')
@@ -419,6 +422,26 @@ sub lookupExpandable {
     return $defn; }
   return; }
 
+# Whether token must be wrapped as dont_expand
+sub isDontExpandable {
+  my ($self, $token) = @_;
+  # Basically: a CS or Active token that is either not defined, or is expandable
+  # (but not \let to a token)
+  return unless $token;
+  my $defn;
+  my $entry;
+  #  my $inmath = $self->lookupValue('IN_MATH');
+  my $cc = $$token[1];
+  if ($CATCODE_ACTIVE_OR_CS[$cc]) {
+    my $lookupname = $$token[0];
+    if ($lookupname
+      && ($entry = $$self{meaning}{$lookupname})
+      && ($defn  = $$entry[0])) {
+      return ((ref $defn) ne 'LaTeXML::Core::Token') && $$defn{isExpandable}; }
+    else {
+      return 1; } }
+  return; }
+
 # used for digestion
 # This recognizes mathactive tokens in math mode
 # and also looks for cs that have been let to other `executable' tokens
@@ -433,16 +456,17 @@ sub lookupDigestableDefinition {
   my $cc   = $$token[1];
   my $name = $$token[0];
   my $lookupname =
-    (($active_or_cs[$cc]
-        || ($letter_or_other[$cc] && $self->lookupValue('IN_MATH')
+    (($CATCODE_ACTIVE_OR_CS[$cc]
+        || ($CATCODE_LETTER_OR_OTHER[$cc] && $self->lookupValue('IN_MATH')
         && (($self->lookupMathcode($name) || 0) == 0x8000)))
     ? $name
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname && ($entry = $$self{meaning}{$lookupname})
     && ($defn = $$entry[0])) {
     # If a cs has been let to an executable token, lookup ITS defn.
     if (((ref $defn) eq 'LaTeXML::Core::Token')
-      && ($lookupname = $executable_primitive_name[$$defn[1]])
+      # If we're digesting an unexpanded, act like \relax
+      && ($lookupname = ($$defn[2] ? '\relax' : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$$defn[1]]))
       && ($entry      = $$self{meaning}{$lookupname})) {
       $defn = $$entry[0]; }
     return $defn; }
@@ -455,7 +479,7 @@ sub installDefinition {
   # Ignore attempts to (re)define $cs from tex sources
   #  my $cs = $definition->getCS->getCSName;
   my $token = $definition->getCS;
-  my $cs = ($LaTeXML::Core::Token::PRIMITIVE_NAME[$$token[1]] || $$token[0]);
+  my $cs    = ($LaTeXML::Core::Token::CATCODE_PRIMITIVE_NAME[$$token[1]] || $$token[0]);
   if ($self->lookupValue("$cs:locked") && !$LaTeXML::Core::State::UNLOCKED) {
     my $s = $self->getStomach->getGullet->getSource;
     # report if the redefinition seems to come from document source
@@ -477,6 +501,33 @@ sub installDefinition {
 
 #======================================================================
 
+# Generate a stub definition for an undefined control-sequence,
+# along with appropriate error messge.
+sub generateErrorStub {
+  my ($self, $caller, $token, $params) = @_;
+  my $cs = $token->getCSName;
+  $self->noteStatus(undefined => $cs);
+  # To minimize chatter, go ahead and define it...
+  if ($cs =~ /^\\if(.*)$/) {    # Apparently an \ifsomething ???
+    my $name = $1;
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as with \\newif");
+    $self->installDefinition(LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'true'), undef, '\let' . $cs . '\iftrue'));
+    $self->installDefinition(LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'false'), undef, '\let' . $cs . '\iffalse'));
+    LaTeXML::Package::Let($token, T_CS('\iffalse')); }
+  else {
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as <ltx:ERROR/>");
+    $self->installDefinition(LaTeXML::Core::Definition::Constructor->new($token, $params,
+        sub { $_[0]->makeError('undefined', $cs); },
+        sizer => 'X'),
+      'global'); }
+  return $token; }
+
+#======================================================================
+
 sub pushFrame {
   my ($self, $nobox) = @_;
   # Easy: just push a new undo hash.
@@ -486,7 +537,7 @@ sub pushFrame {
 sub popFrame {
   my ($self) = @_;
   if ($$self{undo}[0]{_FRAME_LOCK_}) {
-    Fatal('unexpected', '<endgroup>', $self->getStomach,
+    Error('unexpected', '<endgroup>', $self->getStomach,
       "Attempt to pop last locked stack frame"); }
   else {
     my $undo = shift(@{ $$self{undo} });
@@ -541,7 +592,7 @@ sub pushDaemonFrame {
         my $type  = ref $value;
         if (($type eq 'HASH') || ($type eq 'ARRAY')) {    # Only concerned with mutable perl data?
                                                           # Local assignment
-          $$frame{$table}{$key} = 1;                      # Note new value in this frame.
+          $$frame{$table}{$key} = 1;                                  # Note new value in this frame.
           unshift(@{ $$hash{$key} }, daemon_copy($value)); } } } }    # And push new binding.
       # Record the contents of LaTeXML::Package::Pool as preloaded
   my $pool_preloaded_hash = { map { $_ => 1 } keys %LaTeXML::Package::Pool:: };
@@ -574,7 +625,7 @@ sub popDaemonFrame {
       unless (exists $$pool_preloaded_hash{$subname}) {
         undef $LaTeXML::Package::Pool::{$subname};
         delete $LaTeXML::Package::Pool::{$subname};
-      } }
+    } }
     # Finally, pop the frame
     $self->popFrame; }
   else {
@@ -611,7 +662,7 @@ sub activateScope {
         # Here we ALWAYS push the stashed values into the table
         # since they may be popped off by deactivateScope
         my ($table, $key, $value) = @$entry;
-        $$frame{$table}{$key}++;    # Note that this many values must be undone
+        $$frame{$table}{$key}++;                             # Note that this many values must be undone
         unshift(@{ $$self{$table}{$key} }, $value); } } }    # And push new binding.
   return; }
 
@@ -649,26 +700,21 @@ sub getActiveScopes {
 
 #======================================================================
 # Units.
-#   Put here since it could concievably evolve to depend on the current font.
-
-# Conversion to scaled points
-my %UNITS = (    # [CONSTANT]
-  pt => 65536, pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
-  cm => 72.27 * 65536 / 2.54, mm => 72.27 * 65536 / 2.54 / 10, dd => 1238 * 65536 / 1157,
-  cc => 12 * 1238 * 65536 / 1157, sp => 1);
 
 sub convertUnit {
   my ($self, $unit) = @_;
   $unit = lc($unit);
+  # Put here since it could concievably evolve to depend on the current font.
   # Eventually try to track font size?
   if    ($unit eq 'em') { return 10.0 * 65536; }
   elsif ($unit eq 'ex') { return 4.3 * 65536; }
   elsif ($unit eq 'mu') { return 10.0 * 65536 / 18; }
   else {
-    my $sp = $UNITS{$unit};
+    my $units = $self->lookupValue('UNITS');
+    my $sp    = $$units{$unit};
     if (!$sp) {
       Warn('expected', '<unit>', undef, "Illegal unit of measure '$unit', assuming pt.");
-      $sp = $UNITS{'pt'}; }
+      $sp = $$units{'pt'}; }
     return $sp; } }
 
 #======================================================================
@@ -687,26 +733,42 @@ sub getStatus {
   my ($self, $type) = @_;
   return $$self{status}{$type}; }
 
+our $SUCCESS_MESSAGE = 'No obvious problems';
+
 sub getStatusMessage {
   my ($self) = @_;
   my $status = $$self{status};
   my @report = ();
-  push(@report, colorizeString("$$status{warning} warning" . ($$status{warning} > 1 ? 's' : ''), 'warning'))
-    if $$status{warning};
-  push(@report, colorizeString("$$status{error} error" . ($$status{error} > 1 ? 's' : ''), 'error'))
-    if $$status{error};
-  push(@report, "$$status{fatal} fatal error" . ($$status{fatal} > 1 ? 's' : ''))
-
-    if $$status{fatal};
-  my @undef = ($$status{undefined} ? keys %{ $$status{undefined} } : ());
-  push(@report, colorizeString(scalar(@undef) . " undefined macro" . (@undef > 1 ? 's' : '')
-        . "[" . join(', ', @undef) . "]", 'details'))
-    if @undef;
+  my $warning_status = $$status{warning} && colorizeString("$$status{warning} warning" . ($$status{warning} > 1 ? 's' : ''), 'warning');
+  my $error_status = $$status{error} && colorizeString("$$status{error} error" . ($$status{error} > 1 ? 's' : ''), 'error');
+  my $fatal_status = $$status{fatal} && colorizeString("$$status{fatal} fatal error" . ($$status{fatal} > 1 ? 's' : ''), 'fatal');
+  my @undef        = ($$status{undefined} ? keys %{ $$status{undefined} } : ());
+  my $undef_status = @undef && colorizeString(scalar(@undef) . " undefined macro" . (@undef > 1 ? 's' : '')
+      . "[" . join(', ', @undef) . "]", 'details');
   my @miss = ($$status{missing} ? keys %{ $$status{missing} } : ());
-  push(@report, colorizeString(scalar(@miss) . " missing file" . (@miss > 1 ? 's' : '')
-        . "[" . join(', ', @miss) . "]", 'details'))
-    if @miss;
-  return join('; ', @report) || colorizeString('No obvious problems', 'success'); }
+  my $missing_status = @miss && colorizeString(scalar(@miss) . " missing file" . (@miss > 1 ? 's' : '')
+      . "[" . join(', ', @miss) . "]", 'details');
+
+  my $success_status = $SUCCESS_MESSAGE;
+  if ($LaTeXML::Common::Error::IS_TERMINAL) {
+    $warning_status = $warning_status && colorizeString($warning_status, 'warning');
+    $error_status   = $error_status   && colorizeString($error_status,   'error');
+    $fatal_status   = $fatal_status   && colorizeString($fatal_status,   'fatal');
+    $undef_status   = $undef_status   && colorizeString($undef_status,   'details');
+    $missing_status = $missing_status && colorizeString($missing_status, 'details');
+    $success_status = colorizeString($success_status, 'success'); }
+
+  push(@report, $warning_status) if $warning_status;
+  push(@report, $error_status)   if $error_status;
+  push(@report, $fatal_status)   if $fatal_status;
+  push(@report, $undef_status)   if $undef_status;
+  push(@report, $missing_status) if $missing_status;
+
+  my $message = join('; ', @report) || $success_status;
+  $message .= " (See $LaTeXML::Common::Error::LOG_PATH)"
+    if ($$status{fatal} || $$status{error} || $$status{warning})
+    && $LaTeXML::Common::Error::LOG && !ref $LaTeXML::Common::Error::LOG_PATH;
+  return $message; }
 
 sub getStatusCode {
   my ($self) = @_;
@@ -726,7 +788,7 @@ sub getStatusCode {
 
 __END__
 
-=pod 
+=pod
 
 =head1 NAME
 
@@ -736,7 +798,7 @@ C<LaTeXML::Core::State> - stores the current state of processing.
 
 A C<LaTeXML::Core::State> object stores the current state of processing.
 It recording catcodes, variables values, definitions and so forth,
-as well as mimicing TeX's scoping rules.
+as well as mimicking TeX's scoping rules.
 
 =head2 Access to State and Processing
 
@@ -755,7 +817,7 @@ Returns the current Model representing the document model.
 =head2 Scoping
 
 The assignment methods, described below, generally take a C<$scope> argument, which
-determines how the assignment is made.  The allowed values and thier implications are:
+determines how the assignment is made.  The allowed values and their implications are:
 
  global   : global assignment.
  local    : local assignment, within the current grouping.
@@ -803,10 +865,8 @@ Assign $value to be associated with the the string C<$name>, according
 to the given scoping rule.
 
 Values are also used to specify most configuration parameters (which can
-therefor also be scoped).  The recognized configuration parameters are:
+therefore also be scoped).  The recognized configuration parameters are:
 
- VERBOSITY         : the level of verbosity for debugging
-                     output, with 0 being default.
  STRICT            : whether errors (eg. undefined macros)
                      are fatal.
  INCLUDE_COMMENTS  : whether to preserve comments in the
