@@ -257,7 +257,7 @@ sub daemon_ok {
     ['quiet',              ''],
     ['quiet',              ''],
     ['quiet',              ''],
-    ['timeout',            10],
+####    ['timeout',            10],
     ['autoflush',          1],
     ['timestamp',          '0'],
     ['nodefaultresources', ''],
